function [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(GAMMA0,GAMMAj)
% DESCRIPTION
% This function computes the non-null parts of j-th order autocovariance 
% matrix of the powers of the Wiener-Kolmogorov filter necessary for the
% testing procedure proposed in the paper.
%
% INPUT
% GAMMA0 is the (M+K,M+K) 0-th order autocovariance matrix of WK_(t|inf),
% GAMMA0 = E[WK_(t|inf)WK_(t|inf)'],
%
% OUTPUT
% CMe1e1 = cov[m_(1,t|inf),m_(1,t-j|inf)],
% CMe2e2 = cov[m_(2,t|inf),m_(2,t-j|inf)],
% CMe3e3 = cov[m_(3,t|inf),m_(3,t-j|inf)],
% CMe4e4 = cov[m_(4,t|inf),m_(4,t-j|inf)],
% Formulas are given in the paper.
% 
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Computation of non-null parts of autocovariance matrix

K = size(GAMMA0,1);

CMe1e1 = GAMMAj;                                                           % cov(m_(1,t|inf),m_(1,t-j|inf)), dimension (K,K)

CMe2e2 = (kron(ones(K,K),GAMMAj)) .* ...                                
             (kron(GAMMAj,ones(K,K))) + ...
         (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1))));                     % cov(m_(2,t|inf),m_(2,t-j|inf)), dimension (K^2,K^2)

CMe2e4 = 4 * (kron(ones(K^2,1),(reshape(GAMMA0,[K^2,1])'))) .* ...      
             CMe2e2 + ...
         2 * (kron(ones(K^2,1),kron(ones(1,K),(diag(GAMMA0)')))) .* ...
             (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) .* ...
             (kron(GAMMAj,ones(K,K))) + ...
         2 * (kron(ones(K^2,1),kron((diag(GAMMA0)'),ones(1,K)))) .* ...
             (kron(ones(K,K),GAMMAj)) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1))));                     % cov(m_(2,t|inf),m_(4,t-j|inf)), dimension (K^2,K^2)

CMe3e3 = (kron(kron(diag(GAMMA0),ones(K,1)),ones(1,K^2))) .* ...        
             (kron(ones(K^2,1),kron((diag(GAMMA0)'),ones(1,K)))) .* ...
             (kron(ones(K,K),GAMMAj)) + ...
         2 * (kron(ones(K,K),GAMMAj)) .* ...
             (kron((GAMMAj.*GAMMAj),ones(K,K))) + ...
         2 * (kron(reshape(GAMMA0,[K^2,1]),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),kron((diag(GAMMA0)'),ones(1,K)))) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1)))) + ...
         2 * (kron(kron(diag(GAMMA0),ones(K,1)),ones(1,K^2))) .*...
             (kron(ones(K^2,1),(reshape(GAMMA0,[K^2,1])'))) .* ...
             (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) + ...
         4 * (kron(reshape(GAMMA0,[K^2,1]),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),(reshape(GAMMA0,[K^2,1])'))) .* ...
             (kron(GAMMAj,ones(K,K))) + ...
         4 * (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1)))) .* ...
             (kron(GAMMAj,ones(K,K)));                                     % cov(m_(3,t|inf),m_(3,t-j|inf)), dimension (K^2,K^2)

CMe4e4 = 4 * CMe2e2 .* CMe2e2 + ...                                        
         4 * (kron(reshape(GAMMA0,[K^2,1]),ones(1,K^2))) .* ...
             CMe2e4 + ...
         2 * (kron(kron(ones(K,1),diag(GAMMA0)),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),kron(ones(1,K),(diag(GAMMA0)')))) .* ...
             (kron(GAMMAj,ones(K,K))) .* ...
             (kron(GAMMAj,ones(K,K))) + ...
         2 * (kron(kron(ones(K,1),diag(GAMMA0)),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),kron((diag(GAMMA0)'),ones(1,K)))) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1)))) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1)))) + ...
         2 * (kron(kron(diag(GAMMA0),ones(K,1)),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),kron(ones(1,K),(diag(GAMMA0)')))) .* ...
             (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) .* ...
             (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) + ...
         2 * (kron(kron(diag(GAMMA0),ones(K,1)),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),kron((diag(GAMMA0)'),ones(1,K)))) .* ...
             (kron(ones(K,K),GAMMAj)) .* ...
             (kron(ones(K,K),GAMMAj)) + ...
         8 * (kron(kron(ones(K,1),diag(GAMMA0)),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),(reshape(GAMMA0,[K^2,1])'))) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1)))) .* ...
             (kron(GAMMAj,ones(K,K))) + ...
         8 * (kron(kron(diag(GAMMA0),ones(K,1)),ones(1,K^2))) .* ...
             (kron(ones(K^2,1),(reshape(GAMMA0,[K^2,1])'))) .* ...
             (kron(ones(K,K),GAMMAj)) .* ...
             (kron(ones(K,1),kron(GAMMAj,ones(1,K)))) + ...
         8 * (kron(GAMMAj,ones(K,K))) .* ...
             (kron(ones(K,K),GAMMAj)) .* ...
             (kron(ones(1,K),kron(GAMMAj,ones(K,1)))) .* ...
             (kron(ones(K,1),kron(GAMMAj,ones(1,K))));                     % cov(m_(4,t|inf),m_(4,t-j|inf)), dimension (K^2,K^2)