function [s,b,m] = ScoreSS(e,V)
% DESCRIPTION
% This function computes the skewness component of the smoothed score with
% respect to the shape parameters.
%
% INPUT
% e = [e_(1|T) ... e_(T|T)] is (r,T) and contains smoothed errors that are
% non-Gaussian under the alternative, e_(t|T) = LSP[e_t|y_1 ... y_T],
% V = [V_(1|T) ... V_(T|T)] is (r,r,T) and contains the smoother
% mean squared errors, V_(t|T) = E[(e_t-e_(t|T))(e_t-e_(t|T))'].
%
% OUTPUT
% s = [s_(1|T) ... s_(T|T)] is (r,T) and contains smoothed contributions
% to the skewness component of the score with respect to shape parameters
% in each of the asymmetry directions,
% b = [b_(1|T) ... b_(T|T)] is (r+r^2,r,T) coefficients, formula given in
% the paper,
% m = [m_(1|T) ... m_(T|T)] is (r+r^2,T) powers of e, formula given in 
% the paper,
% s_(t|T) = b_(t|T)'m_(t|T) for all t = 1 ... T.
%
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Settings and pre-allocation

[r,T] = size(e);

c3 = -(r+2);

s = zeros(r,T);
b = zeros(r*(r+1),r,T);
m = zeros(r*(r+1),T);


%% 02. Computation of the smoothed contributions to the skewness-score

for t = 1:T
    b1 = (trace(V(:,:,t)) + c3)*eye(r) + 2*V(:,:,t)';
    b3 = kron(ones(r,1),eye(r));
    m3 = reshape((e(:,t))*(e(:,t).^2)',[r^2,1]);
    b(:,:,t) = [b1;b3];
    m(:,t) = [e(:,t);m3];
    s(:,t) = b(:,:,t)'*m(:,t);
end