function GAMMA = AutocovarianceWK(PHI,THETA,SIGMA,j)
% DESCRIPTION
% This function computes the j-th order autocovariance matrix of the
% Wiener-Kolmogorov filter W_(t|inf) from its AR-VMA representation,
%
% (1-Phi(L)) WK_(t|inf) = THETA(L) e_t 
%
% WK_(t|inf) is Wiener-Kolmogorov filter of dimension (H,1),
% e_t is error vector of dimension (K,1), conditionally N(0(K,1),SIGMA).
%
% In the paper this is intended to handle WK_(t|inf) = e_(t|inf) or
% WK_(t|inf) = [X_(t-1|inf)' e_(t|inf)']'.
%
% Explosive roots in (1-Phi(L)) are allowed but no unit root is permitted.
% Of course, there is no room for zero roots of (1-Phi(L)).
%
% INPUT
% PHI is (1,p) and contains the coefficients of the (scalar) AR operator,
% PHI = [Phi1 ... Phip]
% (1-Phi(L)) = 1 - Phi1*L - ... - Phip*L^p,
% THETA is (H,K(q+1)) and contains the coefficients of the MA operator,
% THETA = [THETA0 THETA1 ... THETAq],
% THETA(L) = THETA0 + THETA1*L + ... + THETAq*L^q,
% SIGMA is (K,K) variance matrix of e_t,
% j is the integer order of the autocovariance matrix.
%
% OUTPUT
% GAMMA is the j-th order autocovariance matrix of WK_(t|inf),
% GAMMA = E[WK_(t|inf)WK_(t-j|inf)'].
% 
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Settings and pre-allocation

jlag = abs(j);
H = size(THETA,1);
K = size(SIGMA,1);
p = size(PHI,2);
q = size(THETA,2)/K - 1;
d = H*p + K*q;

auxPHI = [PHI; eye(p-1),zeros(p-1,1)];
auxTHETA0 = THETA(:,1:K);
auxTHETA = THETA(:,K+1:end);
e1 = [1;zeros(p-1,1)];
Jq = [zeros(1,q); eye(q-1),zeros(q-1,1)];

A = [kron(auxPHI,eye(H)),kron(e1,auxTHETA);...                             % Companion matrix of Hyper-VAR representation
     zeros(K*q,H*p),kron(Jq,eye(K))];
Q = zeros(H*p+K*q,H*p+K*q);                                                % Variance matrix of Hyper-VAR representation
Q(1:H,1:H) = auxTHETA0*SIGMA*(auxTHETA0');
Q(1:H,H*p+1:H*p+K) = auxTHETA0*SIGMA;
Q(H*p+1:H*p+K,1:H) = SIGMA*(auxTHETA0');
Q(H*p+1:H*p+K,H*p+1:H*p+K) = SIGMA;


%% 02. Computation of roots

[auxC,auxLambda] = eig(auxPHI);                                         % Jordan decomposition of AR matrix operator
[auxD,auxI] = sort(abs(diag(auxLambda)),'descend');                        % Sorting roots in decreasing orden of absolute value
auxLambda = auxLambda(:,auxI);                                             % Re-ordering eigenvalue matrix
auxLambda = auxLambda(auxI,:);
auxC = auxC(:,auxI);                                                       % Re-ordering eigenvector matrix
auxD = (abs(auxD)>1);
nU = H*sum(auxD);                                                          % Number of unstable roots
nS = H*length(auxD)-nU+K*q;                                                % Number of stable roots


if nU == 0
%% 03. Computation of autocovariance matrix if all roots are stable

    aux0 = (eye(d^2)-kron(A,A)) \ reshape(Q,[d^2,1]);
    aux1 = reshape(aux0,[d,d]);
    
    aux2 = [eye(H),zeros(H,d-H)];                                          % Selection matrix
    
    GAMMA = aux2*(A^jlag)*aux1*aux2';                                      % Autocovariance matrix of order abs(j) of WK_(t|inf)

    
else
%% 04. Quasi-Jordan decomposition of Hyper-VAR companion matrix

    Psi = zeros(H*p,K*q);                                                  % Auxiliary matrix, formula in paper
    for h = 1:q;
        Psi = Psi + kron(auxPHI^(q-h)*e1,auxTHETA) * kron(Jq^(h-1),eye(K));
    end

    Lambda = [kron(auxLambda,eye(H)),zeros(H*p,K*q);...                    % Eigenvalue matrix of Quasi-Jordan form, formula in paper
              zeros(K*q,H*p),kron(Jq,eye(K))];
    C = [kron(auxC,eye(H)),-kron(auxPHI^(-q),eye(H))*Psi;...               % Eigenvector matrix of Quasi-Jordan form, formula in paper
         zeros(K*q,H*p),eye(K*q)];          
    Cinv = [kron(inv(auxC),eye(H)),...                                    % Inverse eigenvector matrix of Quasi-Jordan form, formula in paper
            kron((auxC)\auxPHI^(-q),eye(H))*Psi;...
            zeros(K*q,H*p),eye(K*q)];          

        
%% 05. Computation of autocovariance matrix with unstable roots
        
    R = Cinv*Q*(Cinv');                                                    % Variance matrix of transformed system
    RUU = R(1:nU,1:nU);                                                    % Unstable block of variance matrix
    RSS = R(nU+1:end,nU+1:end);                                            % Stable block of variance matrix
    RSU = R(nU+1:end,1:nU);                                                % Covariance between stable block and unstable block
    LUU = Lambda(1:nU,1:nU);                                               % Unstable block of Hyper-VAR companion matrix of transformed system
    LSS = Lambda(nU+1:end,nU+1:end);                                       % Stable block of Hyper-VAR companion matrix  of transformed system
    LUUinv = (LUU)\eye(size(LUU));                                                    % Inverse of unstable block of Hyper-VAR companion matrix
    
    aux0UU = (eye(nU^2)-kron(conj(LUUinv),LUUinv)) \ ...
        reshape((LUUinv*RUU*LUUinv'),[nU^2,1]);
    aux1UU = reshape(aux0UU,[nU,nU]);
    GAMMAUU = (LUUinv^jlag * aux1UU)';                                     % Computation of autocovariance of unstable block

    aux0SS = (eye(nS^2)-kron(conj(LSS),LSS)) \ ...
        reshape(RSS,[nS^2,1]);
    aux1SS = reshape(aux0SS,[nS,nS]);
    GAMMASS = LSS^jlag * aux1SS;                                           % Computation of autocovariance of stable block
    
    GAMMAUS = 0 * RSU';                                                    % Computation of autocovariance of unstable with stable block
    GAMMASU = 0 * RSU;                                                     % Computation of autocovariance of stable with unstable block
    if jlag > 0
        for h = 1:jlag
            GAMMASU = GAMMASU - (LSS')^(jlag-h)*RSU*(LUUinv')^h;
        end
    end
    
    aux2 = [eye(H),zeros(H,d-H)];
    
    GAMMA = [GAMMAUU,GAMMAUS;GAMMASU,GAMMASS];
    GAMMA = real(aux2*C*GAMMA*C'*aux2');
end


%% 06. Transposition for autocovariance of leads

if j < jlag
    GAMMA = GAMMA';
end