clear 
clc
close all
path([pwd, '/subprograms/'], path)


%% SETTINGS AND PREALLOCATION

options = optimoptions('fmincon','Algorithm','interior-point',...
    'GradObj','off','Hessian','off','Display','iter',...
    'MaxFunEvals',1000,'MaxIter',1000,'TolCon',0.0,'TolFun',10e-9);
pause_length = 1;

sub_period = 1;
if (sub_period == 1)
    T0 = 151; T1 = 242;
elseif (sub_period == 2)
    T0 = 151; T1 = 274;
elseif (sub_period == 3)
    T0 = 21; T1 = 274;
end
T = T1-T0+1;

date0 = 1947.125 + 0.25*(T0-1);                                            % Initial date included in the sample
date1 = 1947.125 + 0.25*(T1-1);                                            % Final date included in the sample
dates = linspace(date0,date1,T1-T0+1);

N = 2;                                                                     % Dimension of vector of observable variables
M = 4;                                                                     % Dimension of vector of state variables
K = 3;                                                                     % Dimension of innovation vector

parameters = [0.75,0.18,0.50,-0.60,0.90,0.20,0.05,0.05]';                   % Parameters: drift, delta, alpha_x, alpha_1, alpha_2, gamma_x, gamma_1, gamma_2

quarter = fopen('IO/data/S7_data.txt');
Data = textscan(quarter,'%f %f %f %f',T1);
fclose(quarter);

Y = 100*log([Data{1},Data{2}])';
Y = Y(:,T0:T1);
YYY = Y/100;
Y = Y - Y(1,1)*ones(2,T);                                                  % Data on GDP-GDI expressed in logs and normalised by initial GDP observation

DY = zeros(3,T-1);
for t = 1:(T-1)
    DY(1,t) = Y(1,t+1) - Y(2,t+1);                                         % Statistical discrepancy
    DY(2:3,t) = Y(:,t+1) - Y(:,t);                                         % Growth rates of GDP-GDI respectively
end

subplot(2,2,1), plot(dates,Y')
axis([ceil(date0) floor(date1) 0 1.02*max(max(Y(:,:)))])
legend('100*log(GDP)','100*log(GDI)','Location','northoutside',...
    'Orientation','horizontal')

subplot(2,2,2), plot(dates(2:end),DY(1,:)')
axis([ceil(date0) floor(date1) min(min(DY(1,:))) max(max(DY(1,:)))])
legend('Statistical Discrepancy','Location','northoutside',...
    'Orientation','horizontal')

subplot(2,2,3), plot(dates(2:end),DY(2,:)')
axis([ceil(date0) floor(date1) min(min(DY(2,:))) max(max(DY(2,:)))])
legend('(log) GDP growth','Location','northoutside',...
    'Orientation','horizontal')

subplot(2,2,4), plot(dates(2:end),DY(3,:)')
axis([ceil(date0) floor(date1) min(min(DY(3,:))) max(max(DY(3,:)))])
legend('(log) GDI growth','Location','northoutside',...
    'Orientation','horizontal')

pause(pause_length)


%% MAXIMUM LIKELIHOOD ESTIMATION

lower = [-10.0 -10.0 -.9999 -.9999 -.9999  0.0001  0.0001  0.0001]';
upper = [ 10.0  10.0  .9999  .9999  .9999 10.0000 10.0000 10.0000]';

[estimate_ML,~,~,~,~,Gradient,Hessian] = fmincon('Likelihood',...
    parameters,[],[],[],[],lower,upper,[],options,Y);

[info,info_mean] = information_matrix(estimate_ML(3:end));
Var_ML = [pinv(info_mean),zeros(2,6);...
    zeros(6,2),pinv(info)]/T;

format short
aux_title = 'parameter   estimate    s.e.        p-value';
aux_c1 = ['drift       ';'delta       ';...
    'alpha_x     ';'alpha_1     ';'alpha_2     ';...
    'gamma_x     ';'gamma_1     ';'gamma_2     '];
aux_c2 = ['    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    '];
aux_matrix = [aux_c1,num2str(estimate_ML,'%f'),aux_c2,...
    num2str(sqrt(diag(Var_ML)),'%f'),aux_c2,...
    num2str(1-normcdf(abs(estimate_ML./sqrt(diag(Var_ML)))),...
    '%f')];
display(aux_matrix,aux_title)
pause(pause_length)


%% KALMAN FILTER/SMOOTHER

C = [(1-estimate_ML(3))*estimate_ML(1); ...
     0; ...
     (1-estimate_ML(4))*estimate_ML(2)/2; ...
     -(1-estimate_ML(5))*estimate_ML(2)/2];
F = [[1+estimate_ML(3),-estimate_ML(3);1,0],zeros(2,2); ...
     zeros(2,2),diag([estimate_ML(4),estimate_ML(5)])];
G = [[sqrt(estimate_ML(6));0],zeros(2,2); ...
      zeros(2,1),diag(sqrt([estimate_ML(7),estimate_ML(8)]))];
H = [ones(2,1), zeros(2,1), eye(2)];

kappa = 1e3;
Ex = 0*C;
Vx = [kappa*eye(2),zeros(2,2);...
      zeros(2,2),...
      diag([estimate_ML(7)/(1-estimate_ML(4)^2),...
       estimate_ML(8)/(1-estimate_ML(5)^2)])];

augC = [C; zeros(3,1)];
augF = [F,zeros(4,3);...
        zeros(3,4),zeros(3,3)];
augG = [G;eye(3)];
augH = [H,zeros(2,3)];

augEx = [Ex;zeros(3,1)];
augVx = [Vx,...
    [[[sqrt(estimate_ML(6));0],zeros(2,2)];...
    zeros(2,1),diag(sqrt([estimate_ML(7),estimate_ML(8)]))];...
    [[[sqrt(estimate_ML(6));0],zeros(2,2)];...
    zeros(2,1),diag(sqrt([estimate_ML(7),estimate_ML(8)]))]',...
    eye(3)];

[Xf_ML,Pf_ML,Xs_ML,Ps_ML] = KalmanFS(augEx,augVx,augF,augG,augH,Y,augC);   % Kalman filter/smoother at estimated parameter values of augmented system

subplot(2,2,1), plot(dates,[Y',Xs_ML(1,:)'])
axis([ceil(date0) floor(date1) 0 1.02*max(max(Y(:,:)))])
legend('100*log(GDP)','100*log(GDI)','smoothed CF',...
    'Location','northoutside',...
    'Orientation','horizontal')

subplot(2,2,2), plot(dates(2:end),[DY(2:3,:);...
    Xf_ML(1,2:end)-Xf_ML(1,1:end-1)]')
axis([ceil(date0) floor(date1) ...
    min(min(DY(2:3,:))) max(max(DY(2:3,:)))])
legend('GDP growth','GDI growth','filtered CF growth',...
    'Location','northoutside',...
    'Orientation','horizontal')

subplot(2,2,3), plot(dates(2:end),[DY(2:3,:);...
    Xs_ML(1,2:end)-Xs_ML(1,1:end-1)]')
axis([ceil(date0) floor(date1) ...
    min(min(DY(2:3,:))) max(max(DY(2:3,:)))])
legend('GDP growth','GDI growth','smoothed CF growth',...
    'Location','northoutside',...
    'Orientation','horizontal')

subplot(2,2,4), plot(dates,Xs_ML(3:4,:)')
axis([ceil(date0) floor(date1) ...
    min(min(Xs_ML(3:4,:))) max(max(Xs_ML(3:4,:)))])
legend('smoothed GDP noise','smoothed GDI noise',...
    'Location','northoutside',...
    'Orientation','horizontal')

pause(pause_length)


%% COMPUTATION OF SMOOTHED SCORES

e0_ML = zeros(N,T-2);
P0_ML = zeros(N,N,T-2);
for t = 2:T-1
    e0_ML(:,t-1) = Y(:,t) - augH*Xf_ML(:,t);
    P0_ML(:,:,t-1) = augH*Pf_ML(:,:,t)*augH';
end
e1_ML = Xs_ML(5:7,2:T-1);
P1_ML = Ps_ML(5:7,5:7,2:T-1);
e2_ML = Xs_ML(6:7,2:T-1);
P2_ML = Ps_ML(6:7,6:7,2:T-1);
e3_ML = Xs_ML(5,2:T-1);
P3_ML = Ps_ML(5,5,2:T-1);
e3a_ML = Xs_ML(6,2:T-1);
P3a_ML = Ps_ML(6,6,2:T-1);
e3b_ML = Xs_ML(7,2:T-1);
P3b_ML = Ps_ML(7,7,2:T-1);

sk0 = ScoreFK(e0_ML,P0_ML);
ss0 = ScoreFS(e0_ML,P0_ML);
sk1 = ScoreSK(e1_ML,P1_ML);
ss1 = ScoreSS(e1_ML,P1_ML);
sk2 = ScoreSK(e2_ML,P2_ML);
ss2 = ScoreSS(e2_ML,P2_ML);
sk3 = ScoreSK(e3_ML,P3_ML);
ss3 = ScoreSS(e3_ML,P3_ML);
sk3a = ScoreSK(e3a_ML,P3a_ML);
ss3a = ScoreSS(e3a_ML,P3a_ML);
sk3b = ScoreSK(e3b_ML,P3b_ML);
ss3b = ScoreSS(e3b_ML,P3b_ML);

score0(1) = mean(sk0,2);
score0(2:3) = chol(pinv(mean(P0_ML,3)),'lower')*mean(ss0,2);
score1(1,1) = mean(sk1,2);
score1(2:4,1) = mean(ss1,2);
score2(1,1) = mean(sk2,2);
score2(2:3,1) = mean(ss2,2);
score3(1,1) = mean(sk3,2);
score3(2,1) = mean(ss3,2);
score3a(1,1) = mean(sk3a,2);
score3a(2,1) = mean(ss3a,2);
score3b(1,1) = mean(sk3b,2);
score3b(2,1) = mean(ss3b,2);
HK(1,1) = StatsHKK(e3_ML');
HK(2,1) = StatsHKS(e3_ML');
HKa(1,1) = StatsHKK(e3a_ML');
HKa(2,1) = StatsHKS(e3a_ML');
HKb(1,1) = StatsHKK(e3b_ML');
HKb(2,1) = StatsHKS(e3b_ML');

subplot(2,3,1), plot(dates(2:end-1),sk1')
axis([ceil(date0) floor(date1) ...
    min(min(sk1)) max(max(sk1))])
title('Case 1: Kurtosis'),...
    legend('score',...
    'Location','southoutside',...
    'Orientation','horizontal')

subplot(2,3,2), plot(dates(2:end-1),sk2')
axis([ceil(date0) floor(date1) ...
    min(min(sk2)) max(max(sk2))])
title('Case 2: Kurtosis'),...
    legend('score',...
    'Location','southoutside',...
    'Orientation','horizontal')

subplot(2,3,3), plot(dates(2:end-1),sk3')
axis([ceil(date0) floor(date1) ...
    min(min(sk3)) max(max(sk3))])
title('Case 3: Kurtosis'),...
    legend('score',...
    'Location','southoutside',...
    'Orientation','horizontal')

subplot(2,3,4), plot(dates(2:end-1),ss1')
axis([ceil(date0) floor(date1) ...
    min(min(ss1)) max(max(ss1))])
title('Case 1: Skewness'),...
    legend('CF score','GDP score','GDI score',...
    'Location','southoutside',...
    'Orientation','horizontal')

subplot(2,3,5), plot(dates(2:end-1),ss2')
axis([ceil(date0) floor(date1) ...
    min(min(ss2)) max(max(ss2))])
title('Case 2: Skewness'),...
    legend('GDP score','GDI score',...
    'Location','southoutside',...
    'Orientation','horizontal')

subplot(2,3,6), plot(dates(2:end-1),ss3')
axis([ceil(date0) floor(date1) ...
    min(min(ss3)) max(max(ss3))])
title('Case 3: Skewness'),...
    legend('CF score',...
    'Location','southoutside',...
    'Orientation','horizontal')

pause(pause_length)


%% COMPUTATION OF ASYMPTOTIC VARIANCES

maxiter = 150;
maxerror = 1e4;

ax = estimate_ML(3);
a1 = estimate_ML(4);
a2 = estimate_ML(5);
gx = estimate_ML(6);
g1 = estimate_ML(7);
g2 = estimate_ML(8);

[PHI,THETA] = Bivariate_WKfilter(ax,a1,a2,gx,g1,g2);
auxGAMMA0 = AutocovarianceWK(PHI,THETA,eye(K),0);

r = K;
[~,aux_bs,~] = ScoreSS(zeros(K,1),eye(K) - auxGAMMA0);
[~,aux_bk,~] = ScoreSK(zeros(K,1),eye(K) - auxGAMMA0);

B_e1_1 = aux_bs(1:r,:);
B_e2_1 = aux_bk(2:1+r^2);
B_e3_1 = aux_bs(r+1:end,:);
B_e4_1 = aux_bk(2+r^2:end);

r = 2;
E2 = [zeros(2,1),eye(2)];
[~,aux_bs,~] = ScoreSS(zeros(2,1),eye(2) - auxGAMMA0(2:3,2:3));
[~,aux_bk,~] = ScoreSK(zeros(2,1),eye(2) - auxGAMMA0(2:3,2:3));

B_e1_2 = E2'*aux_bs(1:r,:);
B_e2_2 = kron(E2',E2')*aux_bk(2:1+r^2);
B_e3_2 = kron(E2',E2')*aux_bs(r+1:end,:);
B_e4_2 = kron(E2',E2')*aux_bk(2+r^2:end);

r = 1;
E3 = [1,0,0];
[~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(1,1));
[~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(1,1));

B_e1_3 = E3'*aux_bs(1:r,:);
B_e2_3 = kron(E3',E3')*aux_bk(2:1+r^2);
B_e3_3 = kron(E3',E3')*aux_bs(r+1:end,:);
B_e4_3 = kron(E3',E3')*aux_bk(2+r^2:end);

E3a = [0,1,0];
[~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(2,2));
[~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(2,2));

B_e1_3a = E3a'*aux_bs(1:r,:);
B_e2_3a = kron(E3a',E3a')*aux_bk(2:1+r^2);
B_e3_3a = kron(E3a',E3a')*aux_bs(r+1:end,:);
B_e4_3a = kron(E3a',E3a')*aux_bk(2+r^2:end);

E3b = [0,0,1];
[~,aux_bs,~] = ScoreSS(0,1 - auxGAMMA0(3,3));
[~,aux_bk,~] = ScoreSK(0,1 - auxGAMMA0(3,3));

B_e1_3b = E3b'*aux_bs(1:r,:);
B_e2_3b = kron(E3b',E3b')*aux_bk(2:1+r^2);
B_e3_3b = kron(E3b',E3b')*aux_bs(r+1:end,:);
B_e4_3b = kron(E3b',E3b')*aux_bk(2+r^2:end);

[CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(auxGAMMA0,auxGAMMA0);

AGFe1e1 = CMe1e1;
AGFe2e2 = CMe2e2;
AGFe3e3 = CMe3e3;
AGFe4e4 = CMe4e4;

j = 0;
error = 1;

while ((j < maxiter) && (error >= maxerror))                              

    auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);
    [CMe1e1,CMe2e2,CMe3e3,CMe4e4] = AutocovarianceMM(auxGAMMA0,auxGAMMAj);
    [CMe1e1m,CMe2e2m,CMe3e3m,CMe4e4m] = AutocovarianceMM(auxGAMMA0,auxGAMMAj');
    
    AGFe1e1 = AGFe1e1 + CMe1e1 + CMe1e1m;
    AGFe2e2 = AGFe2e2 + CMe2e2 + CMe2e2m;
    AGFe3e3 = AGFe3e3 + CMe3e3 + CMe3e3m;
    AGFe4e4 = AGFe4e4 + CMe4e4 + CMe4e4m;

    j = j+1;
    
    de1e1 = max(max(abs(CMe1e1 + CMe1e1m)));
    de2e2 = max(max(abs(CMe2e2 + CMe2e2m)));
    de3e3 = max(max(abs(CMe3e3 + CMe3e3m)));
    de4e4 = max(max(abs(CMe4e4 + CMe4e4m)));
    error = max([de1e1,de2e2,de3e3,de4e4]);
    
    display([j,error],'Iteration')
end

Var_kk_1 = (B_e4_1')*AGFe4e4*(B_e4_1) - (B_e2_1')*AGFe2e2*(B_e2_1);
Var_ss_1 = (B_e3_1')*AGFe3e3*(B_e3_1) - (B_e1_1')*AGFe1e1*(B_e1_1);

Var_kk_2 = (B_e4_2')*AGFe4e4*(B_e4_2) - (B_e2_2')*AGFe2e2*(B_e2_2);
Var_ss_2 = (B_e3_2')*AGFe3e3*(B_e3_2) - (B_e1_2')*AGFe1e1*(B_e1_2);

Var_kk_3 = (B_e4_3')*AGFe4e4*(B_e4_3) - (B_e2_3')*AGFe2e2*(B_e2_3);
Var_ss_3 = (B_e3_3')*AGFe3e3*(B_e3_3) - (B_e1_3')*AGFe1e1*(B_e1_3);

Var_kk_3a = (B_e4_3a')*AGFe4e4*(B_e4_3a) - (B_e2_3a')*AGFe2e2*(B_e2_3a);
Var_ss_3a = (B_e3_3a')*AGFe3e3*(B_e3_3a) - (B_e1_3a')*AGFe1e1*(B_e1_3a);

Var_kk_3b = (B_e4_3b')*AGFe4e4*(B_e4_3b) - (B_e2_3b')*AGFe2e2*(B_e2_3b);
Var_ss_3b = (B_e3_3b')*AGFe3e3*(B_e3_3b) - (B_e1_3b')*AGFe1e1*(B_e1_3b);


kappa_k3(:,1) = auxGAMMA0(1,1)^4;
kappa_k3(:,2) = auxGAMMA0(2,2)^4;
kappa_k3(:,3) = auxGAMMA0(3,3)^4;

kappa_s3(:,1) = auxGAMMA0(1,1)^3;
kappa_s3(:,2) = auxGAMMA0(2,2)^3;
kappa_s3(:,3) = auxGAMMA0(3,3)^3;

j = 0;
error = 1;

while ((j < maxiter) && (error >= maxerror))                              
    
    auxGAMMAj = AutocovarianceWK(PHI,THETA,eye(K),j+1);

    kappa_k3(:,1) = kappa_k3(:,1) + 2*auxGAMMAj(1,1)^4;
    kappa_k3(:,2) = kappa_k3(:,2) + 2*auxGAMMAj(2,2)^4;
    kappa_k3(:,3) = kappa_k3(:,3) + 2*auxGAMMAj(3,3)^4;

    kappa_s3(:,1) = kappa_s3(:,1) + 2*auxGAMMAj(1,1)^3;
    kappa_s3(:,2) = kappa_s3(:,2) + 2*auxGAMMAj(2,2)^3;
    kappa_s3(:,3) = kappa_s3(:,3) + 2*auxGAMMAj(3,3)^3;
    
    j = j+1;
    error = max(trace(auxGAMMAj.^4),trace(auxGAMMAj.^3));
    display([j,error],'Iteration')
end

Var_HKk3 = kappa_k3(:,1)/auxGAMMA0(1,1)^4;
Var_HKk3a = kappa_k3(:,2)/auxGAMMA0(2,2)^4;
Var_HKk3b = kappa_k3(:,3)/auxGAMMA0(3,3)^4;

Var_HKs3 = kappa_s3(:,1)/auxGAMMA0(1,1)^3;
Var_HKs3a = kappa_s3(:,2)/auxGAMMA0(2,2)^3;
Var_HKs3b = kappa_s3(:,3)/auxGAMMA0(3,3)^3;


%% COMPUTATION OF TEST STATISTIC

stat_kk_0 = T*(score0(1)^2/(N*(N+2)/2));
stat_ss_0 = T*(score1(2:4)'*score1(2:4))/(2*(N+2));
stat_gh_0 = stat_kk_0 + stat_ss_0;

stat_kk_1 = T*(score1(1)^2/(Var_kk_1));
stat_ss_1 = T*(score1(2:4)'*pinv(Var_ss_1)*score1(2:4));
stat_gh_1 = stat_kk_1 + stat_ss_1;

stat_kk_2 = T*(score2(1)^2/(Var_kk_2));
stat_ss_2 = T*(score2(2:3)'*pinv(Var_ss_2)*score2(2:3));
stat_gh_2 = stat_kk_2 + stat_ss_2;

stat_kk_3 = T*(score3(1)^2/(Var_kk_3));
stat_ss_3 = T*(score3(2)^2/(Var_ss_3));
stat_gh_3 = stat_kk_3 + stat_ss_3;

stat_kk_3a = T*(score3a(1)^2/(Var_kk_3a));
stat_ss_3a = T*(score3a(2)^2/(Var_ss_3a));
stat_gh_3a = stat_kk_3a + stat_ss_3a;

stat_kk_3b = T*(score3b(1)^2/(Var_kk_3b));
stat_ss_3b = T*(score3b(2)^2/(Var_ss_3b));
stat_gh_3b = stat_kk_3b + stat_ss_3b;

stat_kk_4 = stat_kk_3 + stat_kk_3a + stat_kk_3b;
stat_ss_4 = stat_ss_3 + stat_ss_3a + stat_ss_3b;
stat_gh_4 = stat_kk_4 + stat_ss_4;

stat_HKkk_3 = T*(HK(1)^2/(Var_HKk3));
stat_HKss_3 = T*(HK(2)^2/(Var_HKs3));
stat_HKgh_3 = stat_HKkk_3 + stat_HKss_3;

stat_HKkk_3a = T*(HKa(1)^2/(Var_HKk3a));
stat_HKss_3a = T*(HKa(2)^2/(Var_HKs3a));
stat_HKgh_3a = stat_HKkk_3a + stat_HKss_3a;

stat_HKkk_3b = T*(HKb(1)^2/(Var_HKk3b));
stat_HKss_3b = T*(HKb(2)^2/(Var_HKs3b));
stat_HKgh_3b = stat_HKkk_3b + stat_HKss_3b;

format short
display(['Initial date: ',num2str(floor(date0),'%d'),...
    'Q',num2str(4*(date0-floor(date0)+0.125),'%d')])
display(['Final date:   ',num2str(floor(date1),'%d'),...
    'Q',num2str(4*(date1-floor(date1)+0.125),'%d')])
aux_title = 'case        statistic   critical    p-value';
aux_c1 = ['1. kurtosis ';'1. skewness ';'1. all      ';...
    '2. kurtosis ';'2. skewness ';'2. all      ';...
    '3. kurtosis ';'3. skewness ';'3. all      ';...
    '3a kurtosis ';'3a skewness ';'3a all      ';...
    '3b kurtosis ';'3b skewness ';'3b all      ';...
    '4. kurtosis ';'4. skewness ';'4. all      ';...
    'hk kurtosis ';'hk skewness ';'hk all      ';...
    'ha kurtosis ';'ha skewness ';'ha all      ';...
    'hb kurtosis ';'hb skewness ';'hb all      ';...
    '0. kurtosis ';'0. skewness ';'0. all      '];
aux_c2 = ['    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    ';...
    '    ';'    ';'    ';'    ';'    ';'    '];
stats = [stat_kk_1;stat_ss_1;stat_gh_1;...
    stat_kk_2;stat_ss_2;stat_gh_2;...
    stat_kk_3;stat_ss_3;stat_gh_3;...
    stat_kk_3a;stat_ss_3a;stat_gh_3a;...
    stat_kk_3b;stat_ss_3b;stat_gh_3b;...
    stat_kk_4;stat_ss_4;stat_gh_4;...
    stat_HKkk_3;stat_HKss_3;stat_HKgh_3;...
    stat_HKkk_3a;stat_HKss_3a;stat_HKgh_3a;...
    stat_HKkk_3b;stat_HKss_3b;stat_HKgh_3b;...
    stat_kk_0;stat_ss_0;stat_gh_0];
crits = [chi2inv(0.95,1);chi2inv(0.95,3);chi2inv(0.95,4);...
    chi2inv(0.95,1);chi2inv(0.95,2);chi2inv(0.95,3);...
    chi2inv(0.95,1);chi2inv(0.95,1);chi2inv(0.95,2);...
    chi2inv(0.95,1);chi2inv(0.95,1);chi2inv(0.95,2);...
    chi2inv(0.95,1);chi2inv(0.95,1);chi2inv(0.95,2);...
    chi2inv(0.95,3);chi2inv(0.95,3);chi2inv(0.95,6);...
    chi2inv(0.95,1);chi2inv(0.95,1);chi2inv(0.95,2);...
    chi2inv(0.95,1);chi2inv(0.95,1);chi2inv(0.95,2);...
    chi2inv(0.95,1);chi2inv(0.95,1);chi2inv(0.95,2);...
    chi2inv(0.95,1);chi2inv(0.95,2);chi2inv(0.95,3)];
pvals = [1-chi2cdf(stats(1),1);1-chi2cdf(stats(2),3);...
    1-chi2cdf(stats(3),4);...
    1-chi2cdf(stats(4),1);1-chi2cdf(stats(5),2);...
    1-chi2cdf(stats(6),3);...
    1-chi2cdf(stats(7),1);1-chi2cdf(stats(8),1);...
    1-chi2cdf(stats(9),2);...
    1-chi2cdf(stats(10),1);1-chi2cdf(stats(11),1);...
    1-chi2cdf(stats(12),2);...
    1-chi2cdf(stats(13),1);1-chi2cdf(stats(14),1);...
    1-chi2cdf(stats(15),2);...
    1-chi2cdf(stats(16),3);1-chi2cdf(stats(17),3);...
    1-chi2cdf(stats(18),4);...
    1-chi2cdf(stats(19),1);1-chi2cdf(stats(20),1);...
    1-chi2cdf(stats(21),2);...
    1-chi2cdf(stats(22),1);1-chi2cdf(stats(23),1);...
    1-chi2cdf(stats(24),2);...
    1-chi2cdf(stats(25),1);1-chi2cdf(stats(26),1);...
    1-chi2cdf(stats(27),2);...
    1-chi2cdf(stats(28),1);1-chi2cdf(stats(29),2);...
    1-chi2cdf(stats(30),3)];
aux_matrix = [aux_c1,num2str(stats,'%f'),aux_c2,...
    num2str(crits,'%f'),aux_c2,...
    num2str(pvals,'%f')];
display(aux_matrix,aux_title)


%% MCMC ESTIMATION

% Initialize latent variables
kappa = 1e3;
Ex = zeros(M, 1);
Vx = [kappa*(estimate_ML(6)/(1-estimate_ML(3)^2))*ones(2, 2), zeros(2, 2);...
      zeros(2, 2), ...
      diag([estimate_ML(7)/(1-estimate_ML(4)^2),...
            estimate_ML(8)/(1-estimate_ML(5)^2)])];

C = [(1-estimate_ML(3))*estimate_ML(1); ...
     0; ...
     (1-estimate_ML(4))*estimate_ML(2)/2; ...
     -(1-estimate_ML(5))*estimate_ML(2)/2];
F = [[1+estimate_ML(3), -estimate_ML(3); 1, 0], zeros(2,2); ...
     zeros(2,2), diag([estimate_ML(4), estimate_ML(5)])];
G = [[sqrt(estimate_ML(6)); 0], zeros(2,2); ...
      zeros(2,1), diag(sqrt([estimate_ML(7), estimate_ML(8)]))];
H = [ones(2,1), zeros(2,1), eye(2)];

[Xf_ML, Pf_ML, Xs_ML, Ps_ML] = KalmanFS(Ex, Vx, F, G, H, Y, C);

% Gibbs sampler
data_file = ['IO/results/S7_MCMC_AsymT_', num2str(sub_period), '.mat'];
symmetry = 1;

S_MCMC = 500000; store = 2;
thin = 20; safe = floor(0.1*S_MCMC);
method = 'ARMS'; nr = 3-(strcmp(method, 'ARMS'));
sample = (floor(0.1*S_MCMC)):S_MCMC;

h = waitbar(0, ['Gibbs sampler', ' (', method, ') ']);

xi_mv = zeros(M, T, S_MCMC/store);
zeta_sh = zeros(1, T-1, S_MCMC/store);
theta_mv = zeros(8, S_MCMC);
rejection_mv = zeros(3, nr, S_MCMC);
theta_sh = zeros(4, S_MCMC);
rejection_sh = zeros(4, nr, S_MCMC);
time = zeros(1, S_MCMC);

theta_mv_0 = estimate_ML;
theta_sh_0 = [12; 0; 0; 0];
xi_0 = Xs_ML;
epsilon_0 = (pinv(G'*G)*G')*(xi_0(:, 2:T) - C - F*xi_0(:, 1:(T-1)));
zeta_0 = ones(1, T-1);

for s = 1:S_MCMC
    t1 = cputime;
    
    for s_thin = 1:thin    
    % Drawing mixing variables given mv-parameters, state variables and
    % sh-parameters (Mencia-Sentana, 2010, JBES)
    zeta_0 = GIG_sampler(epsilon_0, theta_sh_0);
    
    % Drawing state variables given mv-parameters, sh-parameters and
    % mixing variables (Durbin-Koopman, 2002, BCMA)
    [xi_0, epsilon_0] = SS_sampler(theta_mv_0, theta_sh_0, zeta_0, Y);

    % Drawing sh-parameters given mv-parameters, state variables and
    % mixing variables (Gilks-Best-Tan, 1995, JRSS)      
    [theta_sh_0, reject_sh] = SH_sampler(theta_sh_0, epsilon_0, zeta_0, method, symmetry);
    
    % Drawing mv-parameters given state variables, sh-parameters and
    % mixing variables
    theta_mv_0(1:2) = C_sampler(theta_mv_0, xi_0, theta_sh_0, zeta_0);
    theta_mv_0(3:5) = F_sampler(theta_mv_0, xi_0, theta_sh_0, zeta_0);
    [theta_mv_0(6:8), reject_mv] = G_sampler(theta_mv_0, xi_0, theta_sh_0, zeta_0, method);
    end

    if (rem(s, store) == 0)
        xi_mv(:, :, s/store) = xi_0;
        zeta_sh(:, :, s/store) = zeta_0;
    end
    
    theta_mv(:, s) = theta_mv_0;        
    rejection_mv(:, :, s) = reject_mv; 
    
    theta_sh(:, s) = theta_sh_0;
    rejection_sh(:, :, s) = reject_sh; 
    
    time(1, s) = 100*(cputime-t1);
    waitbar(s/S_MCMC, h)
    
    if (rem(s, safe) == 0)
        save MCMC.mat s xi_mv zeta_sh theta_mv rejection_mv theta_sh rejection_sh time
        disp(s)
    end
    
end
close(h)

movefile('MCMC.mat', data_file)

disp('  ')
disp('SUMMARY OF POSTERIOR DRAWS')
format short
names_sh = ['\nu        '; '\beta_x    '; '\beta_1    '; '\beta_2    '];
names_mv = ['\pi        '; '\delta     '; '\rho_x     '; '\rho_1     '; ...
            '\rho_2     '; '\gamma_x   '; '\gamma_1   '; '\gamma_2   '];
aux_matrix = [[names_sh(:, 2:11); names_mv(:, 2:11)], ...
              num2str(quantile([theta_sh; theta_mv], [0.05, 0.50, 0.95], 2), '%11.4f')];
aux_heading = [repmat('=', [1, size(aux_matrix, 2)]); ...
               'parameter ', ' (0.05)', repmat(' ', [1, size(aux_matrix, 2)-35]), ...
               ' (0.50)    ', ' (0.95)'; ...
               repmat('=', [1, size(aux_matrix, 2)])];
disp(aux_heading)
disp(aux_matrix)
        
axis_limit_sh = [4.0, 100; -5.0, 5.0; -5.0, 5.0; -5.0, 5.0];
axis_limit_mv = [0.0, 2.0; -5.0, 5.0; ...
                 0.0, 1.0;  0.0, 1.0;  0.0, 1.0; ...
                 0.0, 1.0;  0.0, 1.0;  0.0, 1.0];
                  
for p = 1:4
    subplot(3, 4, p), plot(theta_sh(p, sample)), title(names_sh(p, :))
    axis([1 sample(end)-sample(1) axis_limit_sh(p, 1) axis_limit_sh(p, 2)])
end
for p = 1:8
    subplot(3, 4, 4+p), plot(theta_mv(p, sample)), title(names_mv(p, :))
    axis([1 sample(end)-sample(1) axis_limit_mv(p, 1) axis_limit_mv(p, 2)])
end