function [W, hW] = ARMS_pwexp_draw(w_grid, L_grid)
% This function draws from the piecewiese exponencial hull function for the
% adaptive rejection Metropolis sampling method of Gilks, Best & Tan (1995, JRSS).
% This is achieved in two steps:
% 1. Draw the interval from a multinomial distribution;
% 2. Draw the location from a truncated exponential distribution.
%
% INPUTS:
% - w_grid is the grid for the argument
% - L_grid is the grid for the log-density at each argument
%
% OUTPUT:
% - W a draw from the piecewiese exponencial hull function
% - hW is the value of the piecewise exponencial hull function at W
%
% AUTHOR: Tincho Almuzara, DATE: 22-Dec-2017

tolerance_d = 1e-3;
[w_grid, L_grid] = ARMS_hull(w_grid, L_grid);
L_max = max(L_grid);
L_grid = L_grid - L_max;

n_grid = size(w_grid, 1);
d = (L_grid(2:end)-L_grid(1:(end-1))) ./ (w_grid(2:end)-w_grid(1:(end-1)));

Pr = (exp(L_grid(2:end))-exp(L_grid(1:(end-1)))) ./ d;

for i = 1:(n_grid-1)
    if (abs(d(i)) < tolerance_d)
        Pr(i) = exp(L_grid(i))*(w_grid(i+1)-w_grid(i));
    end
end

CDF = [0; cumsum(Pr(1:(end-1)))./sum(Pr)];

U1 = rand(1, 1);
i1 = sum(U1 >= CDF);

U2 = rand(1, 1);
if (abs(d(i1)) < tolerance_d)
    W = w_grid(i1) + (w_grid(i1+1)-w_grid(i1))*U2;
elseif (d(i1) < -tolerance_d) 
    DD = L_grid(i1+1) - L_grid(i1);	
    W = w_grid(i1) + (w_grid(i1+1)-w_grid(i1))*(log(1-U2)/DD - floor(log(1-U2)/DD));
else
    DD = L_grid(i1) - L_grid(i1+1);	
    W = w_grid(i1) + (w_grid(i1+1)-w_grid(i1))*(1 - log(1-U2)/DD + floor(log(1-U2)/DD));
end
W = max(w_grid(i1), min(w_grid(i1+1), W));

hW = L_max + L_grid(i1) + d(i1)*(W-w_grid(i1));

