function [Xf,Pf,Xs,Ps] = KalmanFS(EX1,VX1,F,G,H,Y,C,D,Z)
% DESCRIPTION
% This function evaluates the Kalman filter and smoother algorithm for
% the state-space model,
%
% X_t = C*z_t + F*X_(t-1) + G*e_(t-1)   [transition equation]
% y_t = D*z_t + H*X_t                  [measurement equation]
%
% X_t is state vector of dimension (M,1),
% y_t is observable endogenous vector of dimension (N,1),
% z_t is observable predetermined vector of dimension (L,1),
% e_t is error vector of dimension (K,1), conditionally N(0(K,1),I(K)).
%
% INPUT
% Data runs through t = 1 ... T and is organized row-wise: 
% Y = [y_1 ... y_T] is (N.T) and Z = [z_1 ... z_T] is (L,T).
%
% C, F, G, D and H are matrices of suitable dimension.
%
% EX1 and VX1 are the prior mean and variance of X_1.
%
% Optional arguments are C, D and Z.
%
% OUTPUT
% Xf = [Xf_(1|0) ... Xf_(T|T-1)] is (M,T) and contains the filtered
% projections, Xf_(t|t-1) = E[X_t|y_1 ... y_(t-1)];
% Pf = [Pf_(1|0) ... Pf_(T|T-1)] is (M,M,T) and contains the filter
% mean squared errors, Pf_(t|t-1) = E[(X_t-Xf_(t|t-1))(X_t-Xf_(t|t-1))'];
% Xs = [Xs_(1|T) ... Xs_(T|T)] is (M,T) and contains the smoothed
% projections, Xs_(t|T) = E[X_t|y_1 ... y_T];
% Ps = [Ps_(1|T) ... Ps_(T|T)] is (M,M,T) and contains the smoother
% mean squared errors, Ps_(t|T) = E[(X_t-Xs_(t|T))(X_t-Xs_(t|T))'].
% 
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Settings and pre-allocation

T = size(Y,2);
M = size(F,1);

Xf = zeros(M,T);                                                           % Filtered state vector
Xu = zeros(M,T-1);                                                         % Updated state vector, Xu_{t|t} = E[X_t|y_1 ... y_t]
Xs = zeros(M,T);                                                           % Smoothed state vector
Pf = zeros(M,M,T);                                                         % MSE of filtered state
Pu = zeros(M,M,T-1);                                                       % MSE of updated state, Pu_(t|t) = E[(X_t-Xu_(t|t))(X_t-Xu_(t|t))']
Ps = zeros(M,M,T);                                                         % MSE of smoothed state

if (nargin == 6)
    aux1 = zeros(M,T);
    aux2 = Y;
elseif (nargin == 7)
    aux1 = C*ones(1,T);
    aux2 = Y;
elseif (nargin == 8)
    aux1 = C*ones(1,T);
    aux2 = Y - D*ones(1,T);
else
    aux1 = C*Z;                                                            % Auxiliary data [C*z_1 ... C*z_T]
    aux2 = Y - D*Z;                                                        % Auxiliary data [y_1-D*z_1 ... y_T-D*z_T]
end


%% 02. Kalman filter

Xf(:,1) = EX1;
Pf(:,:,1) = VX1;

for t = 1:T-1
    K = (Pf(:,:,t)*H') * pinv(H*Pf(:,:,t)*H');
    Pu(:,:,t) = (eye(M) - K*H) * Pf(:,:,t);
    Pf(:,:,t+1) = F*Pu(:,:,t)*F' + G*G';
    Xu(:,t) = Xf(:,t) + K*(aux2(:,t) - H*Xf(:,t));
    Xf(:,t+1) = aux1(:,t+1) + F*Xu(:,t);
end


%% 03. Kalman smoother

K = (Pf(:,:,T)*H') * pinv(H*Pf(:,:,T)*H');
Xs(:,T) = Xf(:,T) + K*(aux2(:,T) - H*Xf(:,T));
Ps(:,:,T) = (eye(M) - K*H) * Pf(:,:,T);

for t = 1:T-1
    J = (Pu(:,:,T-t)*F') * pinv(Pf(:,:,T-t+1));
    Ps(:,:,T-t) = Pu(:,:,T-t) - J*(Pf(:,:,T-t+1) - Ps(:,:,T-t+1))*J';
    Xs(:,T-t) = Xu(:,T-t) + J*(Xs(:,T-t+1) - Xf(:,T-t+1));
end