function [s,b,m] = ScoreSK(e,V)
% DESCRIPTION
% This function computes the kurtosis component of the smoothed score with
% respect to the shape parameters.
%
% INPUT
% e = [e_(1|T) ... e_(T|T)] is (r,T) and contains smoothed errors that are
% non-Gaussian under the alternative, e_(t|T) = LSP[e_t|y_1 ... y_T],
% V = [V_(1|T) ... V_(T|T)] is (r,r,T) and contains the smoother
% mean squared errors, V_(t|T) = E[(e_t-e_(t|T))(e_t-e_(t|T))'].
%
% OUTPUT
% s = [s_(1|T) ... s_(T|T)] is (1,T) and contains smoothed contributions
% to the kurtosis component of the score with respect to shape parameters,
% b = [b_(1|T) ... b_(T|T)] is (1+2r^2,T) coefficients, formula given in
% the paper,
% m = [m_(1|T) ... m_(T|T)] is (1+2r^2,T) powers of e, formula given in 
% the paper,
% s_(t|T) = b_(t|T)'m_(t|T) for all t = 1 ... T.
%
% INFORMATION
% Authors: Almuzara, Amengual, Sentana.
% Date: January 2016.


%% 01. Settings and pre-allocation

[r,T] = size(e);

c0 = r*(r+2)/4;
c1 = -(r+2)/2;
c2 = 1/4;

s = zeros(1,T);
b = zeros(1+2*r^2,T);
m = zeros(1+2*r^2,T);


%% 02. Computation of the smoothed contributions to the kurtosis-score

for t = 1:T
    b0 = c0 + c1*trace(V(:,:,t)) + 2*c2*trace(V(:,:,t)^2)...
        + c2*(trace(V(:,:,t)))^2;
    b2 = c1*reshape(eye(r),[r^2,1]) + 4*c2*reshape(V(:,:,t),[r^2,1])...
        + 2*trace(V(:,:,t))*c2*reshape(eye(r),[r^2,1]);
    b4 = c2*ones(r^2,1);
    m2 = reshape(e(:,t)*e(:,t)',[r^2,1]);
    m4 = reshape((e(:,t).^2)*(e(:,t).^2)',[r^2,1]);
    b(:,t) = [b0;b2;b4];
    m(:,t) = [1;m2;m4];
    s(1,t) = b(:,t)'*m(:,t);
end