%/******************************************************************
% *                                                                *
% *  Generalized Inverse Gaussian Distribution - Ratio of Uniforms *
% *                                                                *
% ******************************************************************
% *                                                                *
% * FUNCTION :   - gigru samples a random number from the          *
% *                reparameterised Generalized Inverse Gaussian    *
% *                distribution with parameters l > 0 and b > 0    *
% *                using Ratio of Uniforms method without shift    *
% *                for  l <= 1  and  b <= 1  and shift at mode  m  *
% *                otherwise.                                      *
% * REFERENCES : - J.S. Dagpunar (1989): An easily implemented     *
% *                generalized inverse Gaussian generator,         *
% *                Commun. Statist. Simul. 18(2), 703-710.         *
% *              - K. Lehner (1989): Erzeugung von Zufallszahlen   *
% *                fuer zwei exotische Verteilungen,               *
% *                Diplomarbeit, Techn. Universitaet Graz.         *
% * SUBPROGRAM : - drand(seed) ... (0,1)-Uniform generator with    *
% *                 unsigned long integer *seed.                   *
% *                                                                *
% * Implemented by K. Lehner, 1990                                 *
% * Revised     by F. Niederl, August 1992                         *
% ******************************************************************/


function fgig=gigru(h,b)

drittel =  0.3333333333333333;
pdrittel = 0.037037037037037;

h_setup = -2.0;
b_setup = -2.0;

 if ((h<=1) & (b<=1))
				  %/* NO SHIFT m */

	if (h ~= h_setup) | (b ~= b_setup)
	 
				  %/* Set-up */
	  e = b*b;
	  d = h + 1.0;
	  ym = (-d + sqrt(d*d + e))/b;
	  d = h - 1.0;
	  xm = (d + sqrt(d*d + e))/b;
	  d = 0.5*d;
	  e = -0.25*b;
	  r = xm + 1.0/xm;
	  s = xm*ym;
							%/* a = vplus/uplus */
	  a = exp(-0.5*h*log(s) + 0.5*log(xm/ym) - e*(r - ym - 1.0/ym));
						%/* c = 1/log{sqrt[hx(xm)]} */
	  c = -d* log(xm) - e*r;
								%  /* vminus = 0 */

	  h_setup = h;
	  b_setup = b;
    end %/* End - Setup */

				  %/* Generator */
	 u = rand;%                                        /* U(0/1) */
	 v = rand; %                                       /* U(0/1) */
	 x = a*(v/u);
                  
    while (((log(u)) > (d*log(x) + e*(x + 1.0/x) + c))) %/* Acceptance/Rejection */
	
	 u = rand;                                        %/* U(0/1) */
	 v = rand;                                        %/* U(0/1) */
	 x = a*(v/u);
    end
else
				  %/* SHIFT BY m */

		 if (h ~= h_setup) | (b ~= b_setup)
	 

				  %/* Set-up */
	  hm1 = h - 1.0;
	  hm12 =hm1*0.5;
	  b2 = b*0.25;
	  m = (hm1 + sqrt(hm1*hm1 + b*b))/b;                 %/* Modus      */
	  max = exp(hm12*log(m) - b2*(m + (1.0/m)));        %/* sqrt[hx(m)] */
	  linvmax = log(1.0/max);

					%/* Find the points x1,x2 (-->invy1,invy2) where
					%the hat function touches the density f(x)    */
	  r = (6.0*m + 2.0*h*m - b*m*m + b)/(4.0*m*m);
	  s = (1.0 + h - b*m)/(2.0*m*m);
	  t = b/(-4.0*m*m);
	  p = (3.0*s - r*r)*drittel;
	  q = (2.0*r*r*r)*pdrittel - (r*s)*drittel + t;
	  eta = sqrt(-(p*p*p)*pdrittel);
	  fi = acos(-q/(2.0*eta));
	  fak = 2.0*exp(log(eta)*drittel);
	  y1 = fak*cos(fi*drittel) - r*drittel;
	  y2 = fak*cos(fi*drittel + 2.0*drittel*pi) - r*drittel;
	  invy1 = 1.0/y1;
	  invy2 = 1.0/y2;

	  vplus = exp(linvmax + log(invy1) + hm12*log(invy1 + m)-b2*(invy1 + m + 1.0/(invy1 + m)));
	  vminus = -exp(linvmax + log(-invy2) + hm12*log(invy2 + m)-b2*(invy2 + m + 1.0/(invy2 + m)));
	  vdiff = vplus - vminus;
									%/* uplus = 1 */

	  h_setup = h;
	  b_setup = b;
  end %/* End - Setup */

				  %/* Generator */
		  u = rand;                                  %/* U(0/1)   */
		  v = vminus + rand*vdiff;                   %/* U(v-/v+) */
		  z =v/u;
         
	    while (z < (-m))
		  u = rand;                                  %/* U(0/1)   */
		  v = vminus + rand*vdiff;                   %/* U(v-/v+) */
		  z =v/u;
        end
	  x = z + m;
                  
	  while ((log(u) > (linvmax + hm12*log(x) - b2*(x + 1.0/x))));  % /* Acceptance/Rejection */
		  u = rand;                                  %/* U(0/1)   */
		  v = vminus + rand*vdiff;                   %/* U(v-/v+) */
		  z =v/u;
         
	    while (z < (-m))
		  u = rand;                                  %/* U(0/1)   */
		  v = vminus + rand*vdiff;                   %/* U(v-/v+) */
		  z =v/u;
        end
	  x = z + m;
         end
     end
 fgig=x;
